Complex class
Constructors
- Complex(double re, double im)
- Complex.fromString(String input)
-
factory
- Complex.polar(double modulus, double theta)
-
factory
- Complex.polarDegrees(double modulus, double degrees)
-
factory
Properties
- conjugate → Complex
-
no setter
- hashCode → int
-
The hash code for this object.
no setteroverride
- im → double
-
no setter
- modulus → double
-
no setter
- modulusCalculate → double
-
no setter
- opposite → Complex
-
no setter
- re → double
-
no setter
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- theta1 → double
-
The argument (phase) in radians in the interval (-π, π].
no setter
- theta2 → double
-
The argument (phase) in radians in the interval [0, 2π).
no setter
Methods
-
cos(
) → Complex -
cosh(
) → Complex -
exp(
) → Complex -
inverse(
) → Complex -
log(
) → Complex -
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
nthRoots(
int n) → List< Complex> -
pow(
int n) → Complex -
printAll(
{int? p}) → void -
rotate(
double angleRadians) → Complex -
sin(
) → Complex -
sinh(
) → Complex -
tan(
) → Complex -
tanh(
) → Complex -
toExponentialString(
{bool use2piRange = false, int? precision}) → String -
toString(
{int? precision}) → String -
A string representation of this object.
override
-
toStringTheta(
) → String -
toTrigonometricString(
{bool use2piRange = false, int? precision}) → String
Operators
-
operator *(
Complex other) → Complex -
operator +(
Complex other) → Complex -
operator -(
Complex other) → Complex -
operator /(
Complex other) → Complex -
operator ==(
Object other) → bool -
The equality operator.
override